@@ -87,18 +87,6 @@ is_sorted <- function(x){
   isTRUE(!is.unsorted(x))
 }
 
-list_subset <- function(x, i, default = NA){
-  check_length(default, 1)
-  if (length(x) == 0){
-    first_element <- NULL
-    ptype <- NULL
-  } else {
-    first_element <- x[[1]]
-    ptype <- first_element[0]
-  }
-  cpp_list_subset(x, ptype, as.integer(i), default)
-}
-
 # Very fast unique function
 sort_unique <- function(x, sort = FALSE){
   if (sort){
@@ -108,7 +96,7 @@ sort_unique <- function(x, sort = FALSE){
     sorted <- isTRUE(attr(o, "sorted"))
     subset <- !(length(sorted_starts) == NROW(x) && sorted)
     if (subset){
-      starts <- o[sorted_starts]
+      starts <- cheapr::sset(o, sorted_starts)
     }
   } else {
     starts <- attr(group3(x, starts = TRUE), "starts")
@@ -149,3 +137,49 @@ vec_rename <- function(.x, ...){
   }
   .x
 }
+
+# Almost working
+# Should prob make internal call checking look at environments instead of namespaces
+
+# register_group_unaware_fn <- function(fn, name = deparse(substitute(fn)), env = rlang::caller_env()){
+#   foo(fn, as.symbol(name), env)
+#   cli::cli_inform(
+#     c(
+#       "{cli::col_magenta(name)} has been registered as a group-unaware function",
+#       "To use this function, simply call it by the name given, e.g. `{name}`()"
+#     )
+#   )
+# }
+
+# Insert values into a vector
+
+# vec_insert <- function(x, after, what){
+#   if (length(what) == 1){
+#     what <- cheapr::cheapr_rep_len(what, length(after))
+#   }
+#   if (length(after) != length(what)){
+#     cli::cli_abort("`length(after)` should match `length(what)`")
+#   }
+#   if (any(after > length(x) | after < 0)){
+#     cli::cli_abort("{.arg after} must be in [0, {length(x)}] range")
+#   }
+#   vecs <- cheapr::new_list((length(after) * 2 ) + 1)
+#
+#   after <- cheapr::cheapr_c(0L, sort(as.integer(after)))
+#
+#   k <- 0L
+#   for (i in seq_along(what)){
+#     k <- k + 1L
+#     if ( (after[i + 1L]) > 0L){
+#       vecs[[k]] <- x[(after[i] + 1L):after[i + 1L]]
+#     }
+#     k <- k + 1L
+#     vecs[[k]] <- what[i]
+#   }
+#
+#   if (after[length(after)] < length(x)){
+#     vecs[[k + 1L]] <- x[(after[length(after)]+ 1L):length(x)]
+#   }
+#
+#   cheapr::cheapr_c(.args = vecs)
+# }
