@@ -63,7 +63,9 @@ GRP3 <- function(X, by = NULL, sort = TRUE,
   }
   if (!is.null(out)){
     out[[8L]] <- GRP_starts(out)
-    out <- c(unclass(out), list(locs = NULL))
+    if (!"locs" %in% names(out)){
+      out <- c(unclass(out), list(locs = NULL))
+    }
     if (return.locs){
       out[[10L]] <- GRP_loc(out)
     }
@@ -191,7 +193,7 @@ check_data_GRP_size <- function(x, GRP){
 GRP_duplicated <- function(GRP, all = FALSE){
   sizes <- GRP_group_sizes(GRP)
   group_id <- GRP_group_id(GRP)
-  out <- (sizes > 1L)[group_id]
+  out <- cheapr::sset(sizes > 1L, group_id)
   if (!all){
     out[GRP_starts(GRP)] <- FALSE
   }
@@ -208,7 +210,7 @@ GRP_duplicated <- function(GRP, all = FALSE){
 #   }
 # }
 GRP_which_duplicated <- function(GRP, all = FALSE){
-  which(GRP_duplicated(GRP, all))
+  cheapr::which_(GRP_duplicated(GRP, all))
 }
 sorted_group_starts <- function(group_sizes, init_loc = 1L){
   cpp_sorted_group_starts(as.integer(group_sizes), init_loc)
@@ -229,15 +231,17 @@ GRP_starts <- function(GRP, use.g.names = FALSE){
       }
       # For factors with 0 size, replace calculated group starts with 0
       out[cheapr::which_val(GRP_sizes, 0L)] <- 0L
-    } else {
+    } else if (!is.null(GRP[["order"]])){
       o <- GRP_order(GRP)
       starts <- attr(o, "starts")
       if (collapse::anyv(GRP_sizes, 0L)){
         out <- integer(GRP_n_groups(GRP))
         out[cheapr::which_val(GRP_sizes, 0L, invert = TRUE)] <- o[starts]
       } else {
-        out <- o[starts]
+        out <- cheapr::attrs_rm(cheapr::sset(o, starts), .set = TRUE)
       }
+    } else {
+      out <- cpp_group_starts(GRP_group_id(GRP), GRP_n_groups(GRP))
     }
   }
   if (is.null(out)){
@@ -249,18 +253,14 @@ GRP_starts <- function(GRP, use.g.names = FALSE){
   out
 }
 # Extract group ends from GRP object safely and efficiently
-GRP_ends <- function(GRP, use.g.names = FALSE,
-                     loc = NULL){
-  GRP_sizes <- GRP_group_sizes(GRP)
+GRP_ends <- function(GRP, use.g.names = FALSE){
   if (GRP_is_sorted(GRP)){
-    out <- sorted_group_ends(GRP_sizes)
+    group_sizes <- GRP_group_sizes(GRP)
+    out <- sorted_group_ends(group_sizes)
     # For factors with 0 size, replace 0 with NA
-    out[cheapr::which_val(GRP_sizes, 0L)] <- 0L
+    out[cheapr::which_val(group_sizes, 0L)] <- 0L
   } else {
-    if (is.null(loc)){
-      loc <- GRP_loc(GRP, use.g.names = FALSE)
-    }
-    out <- GRP_loc_ends(loc, GRP_sizes)
+    out <- cpp_group_ends(GRP_group_id(GRP), GRP_n_groups(GRP))
   }
   if (is.null(out)){
     out <- integer()
@@ -342,13 +342,13 @@ group_locs <- function(x){
 # Groups are assumed to be sorted and
 # index locations are also assumed to be sorted
 GRP_loc_starts <- function(loc){
-  cpp_list_subset(loc, integer(), 1L, 0L)
+  cpp_pluck_list_of_integers(loc, 1L, 0L)
 }
 GRP_loc_ends <- function(loc, sizes = NULL){
   if (is.null(sizes)){
     sizes <- cheapr::list_lengths(loc)
   }
-  list_subset(loc, sizes, default = 0L)
+  cpp_pluck_list_of_integers(loc, sizes, 0L)
 }
 GRP_ordered <- function(GRP){
   GRP[["ordered"]]
@@ -384,7 +384,9 @@ GRP_names <- function(GRP, sep = "_", expand = FALSE, force.char = FALSE){
 # Either treats data as 1 big group or
 # Uses dplyr group vars
 grouped_df_as_GRP <- function(data, return.order = TRUE, ...){
-  cpp_grouped_df_as_grp(data)
+  out <- cpp_grouped_df_as_grp(data)
+  out[["locs"]] <- unclass(f_group_rows(data))
+  out
 }
 # Custom GRP method for data frames
 # Group starts is always returned
@@ -394,7 +396,7 @@ df_to_GRP <- function(data, .cols = character(),
                       return.order = TRUE,
                       return.groups = TRUE,
                       return.locs = FALSE){
-  dplyr_groups <- group_vars(data)
+  dplyr_groups <- f_group_vars(data)
   cols <- drop_names(col_select_names(data, .cols = .cols))
   extra_groups <- vec_setdiff(cols, dplyr_groups)
   data2 <- cheapr::sset_df(data, j = cols)
@@ -453,7 +455,7 @@ df_to_GRP <- function(data, .cols = character(),
     }
   }
   if (return.locs){
-    out <- cheapr::list_assign(out, list("locs" = GRP_loc(out)))
+    out <- cheapr::list_assign(out, list(locs = GRP_loc(out)))
   }
   class(out) <- "GRP"
   out
@@ -499,17 +501,6 @@ GRP.integer64 <- function(X, return.groups = TRUE, ...){
   }
   out
 }
-gsplit2 <- function(x = NULL, g = NULL, use.g.names = FALSE, ...){
-  if (is.null(g)){
-    if (is.null(x)){
-      list(integer())
-    } else {
-      list(x)
-    }
-  } else {
-    collapse::gsplit(x, g = g, use.g.names = use.g.names, ...)
-  }
-}
 
 radixorderv2 <- function(x, na.last = TRUE, decreasing = FALSE,
                          starts = FALSE, sort = TRUE, group.sizes = FALSE){
@@ -570,7 +561,7 @@ construct_grouped_df <- function(data, g, group_vars){
     groups <- cheapr::sset_df(cpp_ungroup(data), GRP_starts(g), j = group_vars)
   }
   group_locs <- GRP_loc(g)
-  groups[[".rows"]] <- vctrs_new_list_of(group_locs, integer())
+  groups[[".rows"]] <- as_list_of_ints(group_locs)
   attr(groups, ".drop") <- df_group_by_drop_default(data)
   attr(groups, "ordered") <- group_by_order_default(data)
   out <- data
@@ -688,3 +679,26 @@ grouped_lag <- function(x, n = 1L, fill = NULL, g = NULL, order_by = NULL){
 grouped_lead <- function(x, n = 1L, fill = NULL, g = NULL, order_by = NULL){
   grouped_lag(x, n = -n, fill = fill, g = g, order_by = order_by)
 }
+
+vec_group_split <- function(x, g){
+  if (is.null(g)){
+    list(x)
+  } else {
+    g <- GRP2(g)
+    locs <- GRP_loc(g)
+    cpp_vec_group_split(x, locs)
+  }
+}
+
+gsplit2 <- function(x = NULL, g = NULL){
+  g <- GRP2(g)
+  if (is.null(g)){
+    list(x)
+  } else {
+    if (cpp_is_simple_atomic_vec(x)){
+      collapse::gsplit(x, g = g)
+    } else {
+      vec_group_split(x, g = g)
+    }
+  }
+}
