@@ -10,4 +10,4 @@
 #' A logical of length 1, either `TRUE` or `FALSE`.
 #'
 #' @export
-group_by_order_default <- df_group_by_order_default
+group_by_order_default <- cpp_group_by_order_default
