@@ -15,9 +15,8 @@ df_collapse <- function(data, cols = names(data),
   if (id){
     out <- df_add_col(out, ".group", df_seq_along(out))
   }
-  include_loc <- loc || end
-  if (include_loc){
-    if (add && identical(group_vars(data), cols) &&
+  if (loc){
+    if (add && identical(f_group_vars(data), cols) &&
         order == group_by_order_default(data) &&
         drop == group_by_order_default(data)){
       GRP_loc <- as.list(group_rows(data))
@@ -32,10 +31,7 @@ df_collapse <- function(data, cols = names(data),
     out <- df_add_col(out, ".start", GRP_starts(g))
   }
   if (end){
-    out <- df_add_col(out, ".end", GRP_ends(g, loc = GRP_loc))
-  }
-  if (!loc && include_loc){
-    out <- df_rm_cols(out, ".loc")
+    out <- df_add_col(out, ".end", GRP_ends(g))
   }
   if (size){
     out <- df_add_col(out, ".size", GRP_group_sizes(g))
@@ -132,8 +128,7 @@ GRP_collapse <- function(g,
   if (id){
     out <- cheapr::df_modify(out, list(.group = df_seq_along(out)))
   }
-  include_loc <- loc || end
-  if (include_loc){
+  if (loc){
     GRP_loc <- GRP_loc(g)
     out <- cheapr::df_modify(out, list(.loc = GRP_loc))
   } else {
@@ -143,10 +138,7 @@ GRP_collapse <- function(g,
     out <- cheapr::df_modify(out, list(.start = GRP_starts(g)))
   }
   if (end){
-    out <- cheapr::df_modify(out, list(.end = GRP_ends(g, loc = GRP_loc)))
-  }
-  if (!loc && include_loc){
-    out <- cheapr::df_modify(out, list(.loc = NULL))
+    out <- cheapr::df_modify(out, list(.end = GRP_ends(g)))
   }
   if (size){
     out <- cheapr::df_modify(out, list(.size = GRP_group_sizes(g)))
@@ -216,9 +208,10 @@ construct_dplyr_group_data <- function(g, drop){
     end = FALSE,
     drop = drop
   )
+  rows <- as_list_of_ints(group_data[[".loc"]])
+
   group_data <- cheapr::list_assign(group_data,
-    list(.rows = vctrs_new_list_of(group_data[[".loc"]], integer()),
-         .loc = NULL)
+    list(.rows = rows, .loc = NULL)
   )
   group_data <- list_as_tbl(group_data)
   attr(group_data, ".drop") <- drop
@@ -244,3 +237,16 @@ construct_fastplyr_grouped_df <- function(data, g, drop = df_group_by_drop_defau
   class(data) <- c("fastplyr_grouped_df", "grouped_df", "tbl_df", "tbl", "data.frame")
   data
 }
+
+# Helper to return group keys where they may be available in a GRP
+# which may or may not be `NULL`
+construct_group_keys <- function(data, g){
+
+  if (is.null(g)){
+    f_group_keys(f_ungroup(data))
+  } else if (is.null(GRP_groups(g))){
+    as_tbl(cheapr::sset(f_ungroup(data), GRP_starts(g)))
+  } else {
+    as_tbl(GRP_groups(g))
+  }
+}
