@@ -1,56 +1,4 @@
 
-.optimised_fn_list <- list(
-
-  input_fns = list(
-    base::sum, base::prod, base::mean, stats::median,
-    base::min, base::max,
-    dplyr::first, dplyr::last,
-    stats::sd, stats::var,
-    dplyr::n_distinct, collapse::fndistinct,
-    collapse::fsum, collapse::fprod, collapse::fmean,
-    collapse::fmedian,
-    collapse::fmin, collapse::fmax,
-    collapse::ffirst, collapse::flast,
-    collapse::fsd, collapse::fvar,
-    collapse::fndistinct
-  ),
-  input_fn_nms = c(
-    "sum", "prod", "mean", "median", "min", "max", "first", "last",
-    "sd", "var", "n_distinct", "ndistinct", "fsum", "fprod", "fmean",
-    "fmedian", "fmin", "fmax", "ffirst", "flast", "fsd", "fvar",
-    "fndistinct"
-  ),
-  input_ns = c(
-    "", "", "base", "stats", "", "", "dplyr", "dplyr",
-    "stats", "stats", "dplyr", "collapse", "collapse", "collapse", "collapse",
-    "collapse", "collapse", "collapse", "collapse", "collapse", "collapse", "collapse",
-    "collapse"
-  ),
-  target_fns = list(
-    collapse::fsum, collapse::fprod, collapse::fmean, collapse::fmedian,
-    collapse::fmin, collapse::fmax,
-    grouped_first, grouped_last,
-    collapse::fsd, collapse::fvar,
-    collapse::fndistinct, collapse::fndistinct,
-    collapse::fsum, collapse::fprod, collapse::fmean,
-    collapse::fmedian,
-    collapse::fmin, collapse::fmax,
-    grouped_first, grouped_last,
-    collapse::fsd, collapse::fvar,
-    collapse::fndistinct
-  )
-)
-.optimised_fns_inform <- c(
-  "sum", "prod", "mean", "median", "min", "max", "sd", "var",
-  "dplyr::n", "dplyr::first", "dplyr::last", "dplyr::n_distinct",
-  "dplyr::row_number", "dplyr::lag", "dplyr::lead",
-  "dplyr::cur_group", "dplyr::cur_group_id", "dplyr::cur_group_rows"
-)
-
-is_optimised_call <- function(expr, env = rlang::caller_env()){
-  cpp_is_fn_call(expr, .optimised_fn_list[["input_fn_nms"]],  NULL, env)
-}
-
 # Somewhat safer check of the .by arg
 # e.g mutate(group_by(iris, Species), .by = any_of("okay"))
 # Should not produce an error with this check
@@ -93,422 +41,6 @@ get_groups <- function(data, .by = NULL, named = FALSE){
   }
 }
 
-# Turn character vector into named character vector
-as_named <- function(x){
-  `names<-`(x, cheapr::str_coalesce(names(x), as.character(x)))
-}
-
-call_args <- function(x){
-  if (typeof(x) != "language"){
-    cli::cli_abort("{.arg x} must be a call")
-  }
-  out <- as.list(x)[-1L]
-  names(out) <- names(out) %||% character(length(out))
-  out
-}
-
-tidy_as_list_of <- function (..., .keep_null = FALSE){
-  dots <- list_tidy(...)
-  if (length(dots) == 1 && !is.object(dots[[1L]]) && is.list(dots[[1L]])) {
-      out <- dots[[1L]]
-  }
-  else {
-    out <- dots
-  }
-  if (!.keep_null){
-    out <- cheapr::list_drop_null(out)
-  }
-  out
-}
-
-quo_labels <- function(quos, named = TRUE){
-  out <- vapply(quos, function(x) deparse2(rlang::quo_get_expr(x)), "", USE.NAMES = FALSE)
-  if (named){
-    names(out) <- names(quos)
-    out <- as_named(out)
-  }
-  out
-}
-
-fix_quo_names <- function(quos){
-
-  nms <- names(quos)
-
-  if (is.null(nms)){
-    nms <- quo_labels(quos, named = FALSE)
-  } else {
-    for (i in seq_along(quos)){
-      if (!nzchar(nms[[i]])){
-        expr <- rlang::quo_get_expr(quos[[i]])
-        if (is.symbol(expr)){
-          nms[i] <- rlang::as_string(expr)
-        } else {
-          nms[i] <- deparse2(expr)
-        }
-      }
-    }
-  }
-  names(quos) <- nms
-  quos
-}
-
-unpack_across <- function(quo, data, unpack_default = FALSE){
-
-  expr <- rlang::quo_get_expr(quo)
-  quo_env <- rlang::quo_get_env(quo)
-
-  clean_expr <- match.call(
-    definition = dplyr::across,
-    call = expr,
-    expand.dots = FALSE,
-    envir = quo_env
-  )
-
-  if (!".cols" %in% names(clean_expr)){
-    cli::cli_abort("{.arg .cols} must be supplied in {.fn across}")
-  }
-  unused_args <- vec_setdiff(names(clean_expr)[-1], c(".cols", ".fns", ".names", ".unpack"))
-
-  if (length(unused_args) > 0){
-    cli::cli_abort("{.arg ...} must be unused")
-  }
-
-  across_vars <- clean_expr[[".cols"]]
-  across_fns <- clean_expr[[".fns"]]
-  across_nms <- clean_expr[[".names"]]
-  across_unpack <- clean_expr[[".unpack"]]
-
-  fn_names <- NULL
-  unpack <- FALSE
-
-  if (is.atomic(across_vars)){
-    cols <- unname(col_select_names(data, across_vars))
-  } else if (rlang::is_call(across_vars, ":")){
-    args <- as.list(across_vars)[-1L]
-    if (length(args) == 2 && is.atomic(args[[1L]]) && is.atomic(args[[2L]])){
-      cols <- unname(col_select_names(data, eval(across_vars, envir = quo_env)))
-    } else {
-      cols <- names(tidyselect::eval_select(across_vars, data, env = quo_env))
-    }
-  } else if (is.symbol(across_vars) && (rlang::as_string(across_vars) %in% names(data))){
-    cols <- rlang::as_string(across_vars)
-  } else {
-    cols <- names(tidyselect::eval_select(across_vars, data, env = quo_env))
-  }
-
-  if (rlang::is_call(across_fns, "list")){
-    fn_tree <- as.list(across_fns)[-1L]
-    fn_names <- names(fn_tree) %||% character(length(fn_tree))
-
-    # This line is to not break timeplyr unit tests
-    if (!all(nzchar(fn_names))){
-      fn_names <- cheapr::str_coalesce(
-        fn_names,
-        vapply(fn_tree, deparse2, "", USE.NAMES = FALSE)
-      )
-    }
-  } else if (!".fns" %in% names(clean_expr)){
-    # fn_tree <- list(identity)
-    fn_tree <- list(as.symbol("identity"))
-  } else {
-    fn_tree <- list(across_fns)
-  }
-
-  # Evaluate functions
-
-  for (i in seq_along(fn_tree)){
-    evaluated_fn <- eval(fn_tree[[i]], envir = quo_env)
-    if (!is.function(evaluated_fn)){
-      fn_tree[[i]] <- rlang::as_function(evaluated_fn)
-    }
-  }
-
-  if (".unpack" %in% names(expr)){
-    unpack <- eval(across_unpack, envir = quo_env)
-  } else {
-    unpack <- unpack_default
-  }
-
-  out_names <- across_col_names(cols, .names = across_nms, .fns = fn_names)
-  out_size <- length(out_names)
-
-  # Recycle cols/fns
-  cols <- cheapr::cheapr_rep_each(cols, out_size / length(cols))
-  fn_tree <- rep_len(fn_tree, out_size)
-
-  out <- cheapr::new_list(out_size)
-  names(out) <- out_names
-
-  for (i in seq_along(out)){
-    fn <- fn_tree[[i]]
-    col <- cols[[i]]
-    new_quo <- rlang::new_quosure(rlang::call2(fn, as.symbol(col)), quo_env)
-    set_add_attr(new_quo, ".unpack", unpack)
-    out[[i]] <- new_quo
-  }
-  out
-}
-
-fastplyr_quos <- function(..., .data, .groups = NULL, .named = TRUE, .drop_null = FALSE,
-                          .unpack_default = FALSE, .optimise = FALSE,
-                          .optimise_expand = FALSE){
-
-  out <- rlang::quos(..., .ignore_empty = "all")
-  quo_nms <- attr(out, "names", TRUE)
-  k <- 1L
-  for (i in seq_along(out)){
-    quo <- out[[k]]
-    expr <- rlang::quo_get_expr(quo)
-    env <- rlang::quo_get_env(quo)
-    attr(quo, ".unpack", TRUE) %||% set_add_attr(quo, ".unpack", .unpack_default)
-    if (!nzchar(quo_nms[[k]]) && cpp_is_fn_call(expr, "across", ns = "dplyr", env)){
-      left <- out[seq_len(k - 1L)]
-      left_nms <- quo_nms[seq_len(k - 1L)]
-      unpacked_quos <- unpack_across(quo, .data, unpack_default = .unpack_default)
-      if (k < length(out)){
-        right <- out[seq.int(k + 1L, length(out), 1L)]
-        right_nms <- quo_nms[seq.int(k + 1L, length(quo_nms), 1L)]
-      } else {
-        right <- list()
-        right_nms <- character()
-      }
-      out[[k]] <- NULL
-      out <- c(left, unpacked_quos, right)
-      quo_nms <- c(left_nms, names(unpacked_quos), right_nms)
-      k <- k + length(unpacked_quos)
-    } else if (.named && !nzchar(quo_nms[[k]])){
-      quo_nms[[k]] <- deparse2(rlang::quo_get_expr(quo))
-      k <- k + 1L
-    } else {
-      k <- k + 1L
-    }
-  }
-  names(out) <- quo_nms
-  if (.drop_null){
-    out <- cpp_quos_drop_null(out)
-  }
-
-  optimised <- logical(length(out))
-
-  if (is.null(.groups)){
-    n_groups <- 1L
-    group_vars <- character()
-  } else {
-    n_groups <- GRP_n_groups(.groups)
-    group_vars <- GRP_group_vars(.groups)
-  }
-
-  if (n_groups <= 1){
-    .fastplyr.g <- NULL
-  } else {
-    .fastplyr.g <- .groups
-  }
-  .original_out <- out
-
-  # Second pass to check for optimised calls
-  if (.optimise && getOption("fastplyr.optimise", TRUE)){
-
-    inform <- getOption("fastplyr.inform")
-    if (inform %||% TRUE){
-      cli::cli_inform(
-        c(
-          "!" = "The following functions will be optimised package-wide:",
-          paste0("`", paste(.optimised_fns_inform, sep = "``"), "`"),
-          "",
-          "Optimised expressions are independent from each other and typical data-masking rules may not apply",
-          "",
-          "Run {.run options(fastplyr.optimise = FALSE)} to disable optimisations globally",
-          "",
-          "Run {.run options(fastplyr.inform = FALSE)} to disable this message"
-        ),
-        .frequency = "once", .frequency_id = ".optimise_inform"
-      )
-    }
-
-    if (is.null(inform)){
-      options(fastplyr.inform = FALSE)
-    }
-
-    if (.optimise_expand){
-      TRA <- "replace_fill"
-    } else {
-      TRA <- NULL
-    }
-    for (i in seq_along(out)){
-      quo <- out[[i]]
-      expr <- rlang::quo_get_expr(quo)
-      env <- rlang::quo_get_env(quo)
-
-      if (is_nested_call(expr)) next
-
-      ### Cases when we are just selecting columns
-      ### We can just optimise it away by leaving the expression as is
-      ### and then running it through `eval_optimised_quos()`
-      if (is.name(expr) && rlang::as_string(expr) == quo_nms[[i]]){
-
-      } else if (.optimise_expand && cpp_is_fn_call(expr, "identity", "base", env)){
-
-      } else if (cpp_is_fn_call(expr, "n", "dplyr", env)){
-        expr <- rlang::call2(\(){
-          if (is.null(.fastplyr.g)){
-            out <- df_nrow(.data)
-            if (.optimise_expand){
-              out <- cheapr::cheapr_rep_len(out, out)
-            }
-          } else {
-            out <- GRP_group_sizes(.fastplyr.g)
-            if (.optimise_expand){
-              out <- out[GRP_group_id(.fastplyr.g)]
-            }
-          }
-          out
-        })
-      } else if (.optimise_expand && cpp_is_fn_call(expr, "row_number", "dplyr", env)){
-        expr <- rlang::call2(\(){
-          if (is.null(.fastplyr.g)){
-            seq_len(df_nrow(.data))
-          } else {
-            cpp_row_id(GRP_order(.fastplyr.g), GRP_group_sizes(.fastplyr.g), TRUE)
-          }
-        })
-      } else if (cpp_is_fn_call(expr, "cur_group_id", "dplyr", env)){
-        expr <- rlang::call2(\(){
-          if (is.null(.fastplyr.g)){
-            if (.optimise_expand){
-              cheapr::cheapr_rep_len(1L, df_nrow(.data))
-            } else {
-              if (df_nrow(.data) == 0L){
-                integer()
-              } else {
-                1L
-              }
-            }
-          } else {
-            if (.optimise_expand){
-              GRP_group_id(.fastplyr.g)
-            } else {
-              GRP_group_id(.fastplyr.g)[GRP_starts(.fastplyr.g)]
-            }
-          }
-        })
-      } else if (cpp_is_fn_call(expr, "cur_group", "dplyr", env)){
-        expr <- rlang::call2(\(){
-          if (is.null(.fastplyr.g)){
-            if (.optimise_expand){
-              new_tbl(.nrows = df_nrow(.data))
-            } else {
-              if (df_nrow(.data) == 0L){
-                new_tbl()
-              } else {
-                f_group_keys(.data)
-              }
-            }
-          } else {
-            if (.optimise_expand){
-              as_tbl(
-                cheapr::sset_row(GRP_groups(.fastplyr.g), GRP_group_id(.fastplyr.g))
-              )
-            } else {
-              GRP_groups(.fastplyr.g)
-            }
-          }
-        })
-      } else if (.optimise_expand && cpp_is_fn_call(expr, "cur_group_rows", "dplyr", env)){
-        expr <- rlang::call2(\(){
-          if (is.null(.fastplyr.g)){
-            seq_len(df_nrow(.data))
-          } else {
-            locs <- GRP_loc(.fastplyr.g)
-            out <- cpp_unlist_group_locs(locs, GRP_group_sizes(.fastplyr.g))
-            order <- cpp_orig_order(GRP_group_id(.fastplyr.g),
-                                    GRP_group_sizes(.fastplyr.g))
-            out[order]
-          }
-        })
-      } else if (.optimise_expand && cpp_is_fn_call(expr, "lag", "dplyr", env)){
-        if (!all_blank(vec_setdiff(names(args), c("x", "n", "default", "order_by")))){
-          next
-        }
-        args <- call_args(expr)
-        names(args)[names(args) == "default"] <- "fill"
-        expr <- rlang::call2("grouped_lag", !!!args, g = .fastplyr.g)
-      } else if (.optimise_expand && cpp_is_fn_call(expr, "lead", "dplyr", env)){
-        if (!all_blank(vec_setdiff(names(args), c("x", "n", "default", "order_by")))){
-          next
-        }
-        args <- call_args(expr)
-        names(args)[names(args) == "default"] <- "fill"
-        expr <- rlang::call2("grouped_lead", !!!args, g = .fastplyr.g)
-      } else if (is_optimised_call(expr, env)){
-        args <- call_args(expr)
-        unsupported_args <- vec_setdiff(names(args), c("x", "na.rm", "nthreads"))
-        if (!all_blank(unsupported_args)){
-          cli::cli_warn(c("Unsupported args: {paste(
-                          unsupported_args[nzchar(unsupported_args)],
-                          collapse = ', '
-                          )}",
-                          "Reverting to un-optimised method"))
-          next
-        }
-        if (call_is_namespaced(expr)){
-          fn <- expr[[1]][[3]]
-          ns <- rlang::as_string(expr[[1]][[2]])
-        } else {
-          fn <- expr[[1]]
-          ns <- fun_ns(fn, env)
-        }
-        fn <- rlang::as_string(fn)
-        match_loc <- match(fn, .optimised_fn_list[["input_fn_nms"]])
-        input_ns <- .optimised_fn_list[["input_ns"]][[match_loc]]
-        if (input_ns != ns){
-          next
-        }
-        fn <- .optimised_fn_list[["target_fns"]][[match_loc]]
-        expr <- rlang::call2(fn, !!!args, g = .fastplyr.g, TRA = TRA, use.g.names = FALSE)
-      } else {
-        next
-      }
-      quo <- rlang::new_quosure(expr, env)
-      set_add_attr(quo, ".unpack", attr(out[[i]], ".unpack", TRUE))
-      optimised[i] <- TRUE
-      out[[i]] <- quo
-    }
-  }
-  if (!all(optimised)){
-    ### Calculate group locations here so that
-    ### tidy evaluation can be done
-    ### eval_all_tidy needs a plain or grouped data frame
-    if (!is.null(.groups)){
-      if (identical(f_group_vars(.data), group_vars) &&
-          GRP_is_ordered(.groups) == group_by_order_default(.data)){
-        .groups[["locs"]] <- f_group_rows(.data)
-      } else {
-        .groups[["locs"]] <- GRP_loc(.groups)
-      }
-    }
-  }
-  cheapr::attrs_add(
-    out,
-    .optimised = optimised,
-    .GRP = .groups,
-    .fastplyr_quos = TRUE,
-    .set = TRUE
-  )
-}
-
-should_optimise <- function(GRP){
-  TRUE
-}
-
-are_fastplyr_quos <- function(quos){
-  isTRUE(attr(quos, ".fastplyr_quos", TRUE))
-}
-check_fastplyr_quos <- function(quos){
-  if (!are_fastplyr_quos(quos)){
-    cli::cli_abort("{.arg quos} must be built using {.fn fastplyr_quos}")
-  }
-}
-
 # Tidyselect col positions with names
 tidy_select_pos <- function(data, ..., .cols = NULL){
   data_nms <- names(data)
@@ -548,7 +80,7 @@ tidy_select_names <- function(data, ..., .cols = NULL){
   add_names(names(data)[match(pos, seq_along(data))], names(pos))
 }
 
-# To replace tidy_group_info_datamask in the future
+# workhorse of f_mutate() with metadata
 mutate_summary <- function(.data, ...,
                            .keep = "all",
                            .by = NULL,
@@ -590,7 +122,7 @@ mutate_summary <- function(.data, ...,
       new_data[common_cols]
     )
     changed_cols <- common_cols[cheapr::val_find(changed, FALSE)]
-    used_cols <- cpp_quo_data_vars(quos, .data)
+    used_cols <- quo_vars(quos, rlang::as_data_mask(.data), combine = TRUE)
     used_cols <- c(used_cols, vec_setdiff(new_cols, used_cols))
     unused_cols <- vec_setdiff(original_cols, new_cols)
 
@@ -727,123 +259,130 @@ check_rowwise <- function(data){
   }
 }
 
-sset_quos <- function(quos, i){
-  out <- quos[i]
-  set_add_attr(out, ".optimised", attr(quos, ".optimised", TRUE)[i])
-  cpp_rebuild(
-    out, quos, c("names", "class", ".optimised"),
-    vec_setdiff(names(attributes(quos)), c("names", "class", ".optimised")),
-    FALSE
-  )
-}
-
-eval_optimised_quos <- function(data, quos, add_groups = TRUE){
+unpack_results <- function(x, quos){
 
   quo_names <- names(quos)
-  quo_groups <- attr(quos, ".GRP", TRUE)
-  GRP <- attr(quos, ".GRP", TRUE)
-  if (!all(attr(quos, ".optimised", TRUE))){
-    cli::cli_abort("All quosures must be tagged as '.optimised'")
-  }
-
-  mask <- rlang::as_data_mask(data)
-  results <- lapply(quos, \(x) rlang::eval_tidy(rlang::quo_get_expr(x), mask))
-
-  groups <- NULL
-
-  if (add_groups){
-    if (length(quos) == 0){
-      groups <- list()
-      names(groups) <- character()
-    } else {
-      groups <- cheapr::cheapr_rep_len(
-        list(
-          cheapr::cheapr_rep_len(GRP_groups(GRP), vector_length(results[[1]]))
-        ),
-        length(quos)
-      )
-      names(groups) <- quo_names
-    }
-  }
-  list(groups = groups, results = results)
-}
-
-eval_all_tidy <- function(data, quos, recycle = FALSE){
-  check_fastplyr_quos(quos)
-
-  if (length(quos) == 0L){
-    return(
-      list(groups = `names<-`(list(), character()),
-           results = `names<-`(list(), character()))
-    )
-  }
-
-  GRP <- attr(quos, ".GRP", TRUE)
-
-  quo_names <- names(quos)
-  which_optimised <- cheapr::val_find(attr(quos, ".optimised", TRUE), TRUE)
-
-  if (length(which_optimised) > 0L){
-    which_regular <- cheapr::val_find(attr(quos, ".optimised", FALSE), FALSE)
-    regular_quos <- sset_quos(quos, which_regular)
-    optimised_quos <- sset_quos(quos, which_optimised)
-    regular_results <- eval_all_tidy(data, regular_quos, recycle = FALSE)
-    optimised_results <- eval_optimised_quos(data, optimised_quos, add_groups = TRUE)
-
-    groups <- cheapr::new_list(length(quos))
-    results <- cheapr::new_list(length(quos))
-
-    groups[which_regular] <- regular_results[[1L]]
-    results[which_regular] <- regular_results[[2L]]
-
-    groups[which_optimised] <- optimised_results[[1L]]
-    results[which_optimised] <- optimised_results[[2L]]
-
-    if (recycle){
-      results <- cheapr::recycle(.args = results)
-      groups <- cheapr::cheapr_rep_len(
-        list(cheapr::cheapr_rep_len(groups[[1]], vector_length(results[[1]]))),
-        length(results)
-      )
-    }
-
-    names(groups) <- quo_names
-    names(results) <- quo_names
-
-    return(list(groups = groups, results = results))
-  }
-
-  if (cpp_any_quo_contains_dplyr_mask_call(quos)){
-    return(dplyr_eval_all_tidy(
-      construct_fastplyr_grouped_df(data, GRP), !!!quos
-    ))
-  }
-  all_results <- cpp_grouped_eval_tidy(
-    construct_fastplyr_grouped_df(data, GRP), quos,
-    recycle = recycle, add_groups = TRUE
-  )
-  groups <- all_results[[1L]]
-  results <- all_results[[2L]]
-  n_group_vars <- length(GRP_group_vars(GRP))
 
   k <- 1L
-  for (i in seq_along(results)){
+  for (i in seq_along(x)){
     # Unpack
-    if (isTRUE(attr(quos[[i]], ".unpack", TRUE)) && is_df(results[[k]])){
-      results_to_append <- as.list(results[[k]])
+    if (isTRUE(attr(quos[[i]], ".unpack", TRUE)) && is_df(x[[k]])){
+      results_to_append <- as.list(x[[k]])
       if (nzchar(quo_names[[i]])){
         names(results_to_append) <- paste(quo_names[[i]], names(results_to_append), sep = "_")
       }
-      results <- append(results, results_to_append, after = k - 1L)
+      x <- append(x, results_to_append, after = k - 1L)
       k <- k + length(results_to_append)
-      results[[k]] <- NULL
+      x[[k]] <- NULL
     } else {
       k <- k + 1L
     }
   }
+  x
+}
+
+eval_all_tidy_optimised_quos <- function(data, quos){
+
+  quo_names <- names(quos)
+  quo_groups <- attr(quos, ".GRP", TRUE)
+  GRP <- attr(quos, ".GRP", TRUE)
+  group_unaware <- attr(quos, ".group_unaware", TRUE)
+  if (!all(attr(quos, ".optimised", TRUE))){
+    cli::cli_abort("All quosures must be tagged as '.optimised'")
+  }
+
+  n_quos <- length(quos)
+
+  mask <- rlang::as_data_mask(data)
+  results <- lapply(quos, rlang::eval_tidy, mask)
+
+  group_order <- NULL
+  reorder <- FALSE
+
+  if (any(group_unaware)){
+
+    # This is for expressions like `+`
+    # where we re-order it to match the order of sorted groups
+    # which is the way `reframe()` returns its output
+
+    if (!is.null(GRP) && GRP_n_groups(GRP) > 1){
+      group_order <- GRP_order(GRP)
+      reorder <- !isTRUE(attr(group_order, "sorted"))
+    }
+
+    if (reorder){
+      results[group_unaware] <- lapply(
+        results[group_unaware], \(x) cheapr::sset(x, group_order)
+      )
+    }
+  }
+
+  groups <- cheapr::new_list(n_quos)
+
+  if (any(group_unaware)){
+    reordered_groups <- cheapr::sset_col(data, GRP_group_vars(GRP))
+    if (reorder){
+      reordered_groups <- cheapr::sset(reordered_groups, group_order)
+    }
+
+    reordered_groups <- cheapr::cheapr_rep_len(
+      list(reordered_groups), cheapr::val_count(group_unaware, TRUE)
+    )
+
+  } else {
+    reordered_groups <- list()
+  }
+
+  if (any(!group_unaware)){
+    reframed_groups <- cheapr::cheapr_rep_len(
+      list(
+        cheapr::cheapr_rep_len(
+          GRP_groups(GRP), vector_length(results[cheapr::which_(group_unaware, invert = TRUE)[1]][[1]])
+        )
+      ),
+      cheapr::val_count(group_unaware, FALSE)
+    )
+  } else {
+    reframed_groups <- list()
+  }
+
+  groups[group_unaware] <- reordered_groups
+  groups[!group_unaware] <- reframed_groups
+  names(groups) <- quo_names
   list(groups = groups, results = results)
 }
 
+eval_summarise_optimised_quos <- function(data, quos){
+
+  if (!all(attr(quos, ".optimised", TRUE))){
+    cli::cli_abort("All quosures must be tagged as '.optimised'")
+  }
+
+  n_groups <- GRP_n_groups(attr(quos, ".GRP")) %||% 1
+
+  mask <- rlang::as_data_mask(data)
+
+  out <- lapply(quos, rlang::eval_tidy, mask)
+  nrows <- df_nrow(data)
+
+  for (res in out){
+    size <- vector_length(res)
+    if ( !(size == 0 && nrows == 0 || size == n_groups) ){
+     cli::cli_abort("All optimised expressions should return 1 row per-group")
+    }
+  }
+
+  out
+}
+
+eval_mutate_optimised_quos <- function(data, quos){
+
+  if (!all(attr(quos, ".optimised", TRUE))){
+    cli::cli_abort("All quosures must be tagged as '.optimised'")
+  }
+  cpp_eval_all_tidy(quos, rlang::as_data_mask(data))
+}
+
 ### In the future we hope to not have to use this
 ### But right now I'm not sure how to evaluate
 ### data-mask specific functions like `n()` and friends
@@ -852,7 +391,7 @@ eval_all_tidy <- function(data, quos, recycle = FALSE){
 ### return a structure similar to the one
 ### eval_all_tidy() produces while keeping the expressions
 ### sequentially dependent
-dplyr_eval_all_tidy <- function(data, ...){
+dplyr_eval_reframe <- function(data, ...){
 
   quos <- rlang::enquos(...)
   expr_names <- names(quos)
@@ -876,44 +415,179 @@ dplyr_eval_all_tidy <- function(data, ...){
     }
     result2 <- as.list(result)[seq_len(df_ncol(result) - n_groups) + n_groups]
     results <- c(results, result2)
-    groups <- cheapr::list_combine(groups, rep(list(df_as_tbl(cheapr::sset_col(result, group_vars))), length(result2)))
+    groups <- cheapr::list_combine(groups, cheapr::cheapr_rep(list(df_as_tbl(cheapr::sset_col(result, group_vars))), length(result2)))
   }
   names(groups) <- names(results)
   list(groups = groups, results = results)
 }
 
-eval_mutate <- function(data, quos){
-  check_fastplyr_quos(quos)
+dplyr_eval_summarise <- function(data, ...){
 
-  if (length(quos) == 0){
-    return(`names<-`(list(), character()))
+  quos <- rlang::enquos(...)
+  expr_names <- names(quos)
+  group_vars <- f_group_vars(data)
+  n_groups <- length(group_vars)
+
+  results <- list()
+
+  # Loop over the expressions
+  for (i in seq_along(quos)){
+    quo <- quos[[i]]
+    expr <- rlang::quo_get_expr(quo)
+    expr_name <- expr_names[i]
+    env <- rlang::quo_get_env(quo)
+    #  Fix this later as new objects don't take precedence over data variables here
+    if (nzchar(expr_name)){
+      result <- dplyr::summarise(data, !!expr_name := !!quo)
+    } else {
+      result <- dplyr::summarise(data, !!quo)
+    }
+    result2 <- as.list(result)[seq_len(df_ncol(result) - n_groups) + n_groups]
+    results <- c(results, result2)
   }
+  list(groups = f_group_keys(data), results = results)
+}
+
+eval_all_tidy <- function(data, quos, recycle = FALSE){
+
+  check_fastplyr_quos(quos)
+  quo_names <- names(quos)
 
   GRP <- attr(quos, ".GRP", TRUE)
-  quo_names <- names(quos)
-  which_optimised <- cheapr::val_find(attr(quos, ".optimised", TRUE), TRUE)
+  optimised <- attr(quos, ".optimised", TRUE)
+  group_unaware <- attr(quos, ".group_unaware", TRUE)
 
-  if (length(which_optimised) > 0L){
-    which_regular <- cheapr::val_find(attr(quos, ".optimised", FALSE), FALSE)
-    regular_quos <- sset_quos(quos, which_regular)
-    optimised_quos <- sset_quos(quos, which_optimised)
-    optimised_results <- eval_optimised_quos(data, optimised_quos, add_groups = FALSE)
-    regular_results <- eval_mutate(data, regular_quos)
-    results <- cheapr::new_list(length(quos))
-    results[which_regular] <- regular_results
-    results[which_optimised] <- optimised_results[[2L]]
-    names(results) <- quo_names
-    return(results)
-  }
-
-  if (cpp_any_quo_contains_dplyr_mask_call(quos)){
-    return(as.list(dplyr::mutate(
-      construct_fastplyr_grouped_df(data, GRP), !!!quos
-    ))[quo_names])
-  }
-  results <- cpp_grouped_eval_mutate(
-    construct_fastplyr_grouped_df(data, GRP), quos
+  if (is.null(GRP)){
+    GRP <- df_as_one_GRP(data)
+    cheapr::attrs_modify(
+      quos, .GRP = GRP, .set = TRUE
     )
+  }
+
+  which_optimised <- cheapr::val_find(optimised, TRUE)
+  which_regular <- cheapr::val_find(optimised, FALSE)
+
+  n_regular <- length(which_regular)
+  n_group_aware_optimised <- cheapr::val_count(group_unaware, FALSE)
+  n_group_unaware <- cheapr::val_count(group_unaware, TRUE)
+
+  # If slow_recycle is true then we have to do a manual recycling of results
+  slow_recycle <- recycle && length(which_optimised) > 0 &&
+    length(which_regular) > 0
+
+  slow_recycle <- slow_recycle || (n_group_unaware > 0 && n_group_aware_optimised > 0)
+
+  if (slow_recycle){
+    backup_GRP <- GRP
+
+    GRP[["groups"]] <- cheapr::new_df(
+      .internal.x = cheapr::sset(
+        GRP_group_id(GRP), GRP_starts(GRP)
+      )
+    )
+    GRP[["group.vars"]] <- ".internal.x"
+
+    cheapr::attrs_modify(
+      quos, .GRP = GRP, .set = TRUE
+    )
+    data <- df_add_cols(
+      data, list(.internal.x = GRP_group_id(GRP))
+    )
+  }
+
+  regular_quos <- sset_quos(quos, which_regular)
+  optimised_quos <- sset_quos(quos, which_optimised)
+  optimised_results <- eval_all_tidy_optimised_quos(data, optimised_quos)
+
+  inform_user_on_eval_split(regular_quos, optimised_quos)
+
+  if (cpp_any_quo_contains_dplyr_mask_call(regular_quos)){
+    regular_results <- dplyr_eval_reframe(
+      construct_fastplyr_grouped_df(data, GRP), !!!regular_quos
+    )
+  } else {
+    # Doing this check to potentially avoid calculating group locations
+    # through a call to `construct_fastplyr_grouped_df()`
+    # which will happen in e.g. `f_reframe(df, new = mean(x), .by = g)`
+    if (length(regular_quos) == 0){
+      regular_results <- list(
+        groups = `names<-`(list(), character()),
+        results = `names<-`(list(), character())
+      )
+    } else if (length(optimised_quos) == 0){
+      regular_results <- cpp_grouped_eval_tidy(
+        construct_fastplyr_grouped_df(data, GRP), regular_quos,
+        recycle = recycle, add_groups = TRUE
+      )
+    } else {
+      # Don't recycle in this case as we will manually recycle later
+      regular_results <- cpp_grouped_eval_tidy(
+        construct_fastplyr_grouped_df(data, GRP), regular_quos,
+        recycle = FALSE, add_groups = TRUE
+      )
+    }
+  }
+
+  groups <- cheapr::new_list(length(quos))
+  groups[which_regular] <- regular_results[[1]]
+  groups[which_optimised] <- optimised_results[[1]]
+  names(groups) <- quo_names
+
+  results <- cheapr::new_list(length(quos))
+  results[which_regular] <- regular_results[[2]]
+  results[which_optimised] <- optimised_results[[2]]
+  names(results) <- quo_names
+
+  # If the user provides expressions that can be optimised alongside
+  # ones which can't, then special by-group recycling needs to occur
+  # to ensure that each within-group result is recycled correctly
+
+  if (slow_recycle){
+
+    group_keys <- construct_group_keys(data, backup_GRP)
+    grps <- lapply(groups, \(df) GRP2(
+      df[[1L]], return.locs = FALSE, sort = FALSE, return.order = FALSE,
+      return.groups = FALSE
+    ))
+
+    sizes <- lapply(grps, GRP_group_sizes)
+
+    all_sizes_the_same <- TRUE
+
+    for (i in 2:length(sizes)){
+      all_sizes_the_same <- all_sizes_the_same &&
+        identical(sizes[[i]], sizes[[i - 1L]])
+    }
+
+    if (!all_sizes_the_same){
+
+      # Split results by group
+
+      # Add group locations
+      grps <- lapply(grps, \(x){
+        x[["locs"]] <- GRP_loc(x)
+        x
+      })
+
+      split_results <- purrr::map2(results, grps, \(x, g) gsplit2(x, g = g))
+      split_results <- transpose_eval_results(split_results)
+      split_results <- recycle_eval_results(split_results)
+      split_results <- transpose_eval_results(split_results)
+      recycled_sizes <- cheapr::list_lengths(split_results[[1]])
+      results <- lapply(split_results, \(x) cheapr::cheapr_c(.args = x))
+    } else {
+      recycled_sizes <- sizes[[1]]
+    }
+    # These are group IDs of our recycled groups
+    out_groups <- cheapr::seq_id(recycled_sizes)
+    groups <- cheapr::sset(group_keys, out_groups)
+    groups <- cheapr::cheapr_rep_len(
+      list(groups), length(results)
+    )
+    names(groups) <- quo_names
+  }
+
+  # Unpack results
 
   k <- 1L
   for (i in seq_along(results)){
@@ -924,12 +598,106 @@ eval_mutate <- function(data, quos){
         names(results_to_append) <- paste(quo_names[[i]], names(results_to_append), sep = "_")
       }
       results <- append(results, results_to_append, after = k - 1L)
+      groups <- append(groups, cheapr::cheapr_rep_len(groups[k], length(results_to_append)), after = k - 1L)
+      names(groups) <- names(results)
       k <- k + length(results_to_append)
       results[[k]] <- NULL
+      groups[[k]] <- NULL
     } else {
       k <- k + 1L
     }
   }
+
+  list(groups = groups, results = results)
+}
+
+eval_summarise <- function(data, quos){
+
+  check_fastplyr_quos(quos)
+  quo_names <- names(quos)
+
+  optimised <- attr(quos, ".optimised", TRUE)
+  GRP <- attr(quos, ".GRP", TRUE)
+
+  which_optimised <- cheapr::val_find(optimised, TRUE)
+  which_regular <- cheapr::val_find(optimised, FALSE)
+  regular_quos <- sset_quos(quos, which_regular)
+  optimised_quos <- sset_quos(quos, which_optimised)
+  optimised_results <- eval_summarise_optimised_quos(data, optimised_quos)
+
+  inform_user_on_eval_split(regular_quos, optimised_quos)
+
+  if (cpp_any_quo_contains_dplyr_mask_call(regular_quos)){
+    regular_results <- dplyr_eval_summarise(
+      construct_fastplyr_grouped_df(data, GRP), !!!regular_quos
+    )
+  } else {
+    # Doing this check to potentially avoid calculating group locations
+    # through a call to `construct_fastplyr_grouped_df()`
+    # which will happen in e.g. `f_summarise(df, new = mean(x), .by = g)`
+    if (length(regular_quos) == 0){
+      regular_results <- list(
+        groups = construct_group_keys(data, GRP),
+        results = `names<-`(list(), character())
+      )
+    } else {
+      regular_results <- cpp_grouped_eval_summarise(
+        construct_fastplyr_grouped_df(data, GRP), regular_quos
+      )
+    }
+  }
+
+
+  groups <- regular_results[[1]]
+
+  results <- cheapr::new_list(length(quos))
+  results[which_regular] <- regular_results[[2]]
+  results[which_optimised] <- optimised_results
+  names(results) <- quo_names
+  results <- unpack_results(results, quos)
+  list(groups = groups, results = results)
+}
+
+eval_mutate <- function(data, quos){
+
+  check_fastplyr_quos(quos)
+  quo_names <- names(quos)
+
+  optimised <- attr(quos, ".optimised", TRUE)
+  GRP <- attr(quos, ".GRP", TRUE)
+
+  which_optimised <- cheapr::val_find(optimised, TRUE)
+  which_regular <- cheapr::val_find(optimised, FALSE)
+  regular_quos <- sset_quos(quos, which_regular)
+  optimised_quos <- sset_quos(quos, which_optimised)
+  optimised_results <- eval_mutate_optimised_quos(data, optimised_quos)
+
+  inform_user_on_eval_split(regular_quos, optimised_quos)
+
+  if (cpp_any_quo_contains_dplyr_mask_call(regular_quos)){
+    regular_results <- as.list(dplyr::mutate(
+      construct_fastplyr_grouped_df(data, GRP), !!!regular_quos
+    ))[names(regular_quos)]
+  } else {
+    # Doing this check to potentially avoid calculating group locations
+    # through a call to `construct_fastplyr_grouped_df()`
+    # which will happen in e.g. `f_mutate(df, new = mean(x), .by = g)`
+    if (length(regular_quos) == 0){
+      regular_results <- `names<-`(list(), character())
+    } else {
+      regular_results <- cpp_grouped_eval_mutate(
+        construct_fastplyr_grouped_df(data, GRP), regular_quos
+      )
+    }
+  }
+
+
+
+  results <- cheapr::new_list(length(quos))
+  results[which_regular] <- regular_results
+  results[which_optimised] <- optimised_results
+  names(results) <- quo_names
+  results <- unpack_results(results, quos)
   results
 }
 
@@ -1114,3 +882,7 @@ tidy_group_info <- function(data, ..., .by = NULL, .cols = NULL,
 #   }
 #   invisible(x)
 # }
+
+as_list_of_ints <- function(x){
+  vctrs_new_list_of(x, integer())
+}
